function love.load()
    -- for a better visibility
    love.graphics.setPointSize(3)
    
    reset_everything()
    
    -- set this to your needs
    maximal_volume = 10000  -- (in pixel)
    needed_length = 1000    -- (in pixel)
    maximal_time = 2        -- (in seconds)
end

function love.update(dt)
    if (xmax - xmin)*(ymax - ymin) < maximal_volume then
        is_involume = true
    else
        is_involume = false
    end
    
    stime = stime + dt
    if stime < maximal_time then
        is_in_time = true
    else
        is_in_time = false
    end
    
    if length_moved > needed_length and is_in_time and is_involume then
        is_won = true
    end
end

function love.draw()
    -- print some instructions
    love.graphics.setColor(255, 255, 255)
    love.graphics.print("Instructions: Click a mouse button to reset at the current position.", 10, 10)
    
    -- print some information
    if length_moved < needed_length then
        love.graphics.setColor(255, 0, 0)
    else
        love.graphics.setColor(0, 255, 0)
    end
    love.graphics.print("Lenght moved: " .. string.format("%.0f", 100*length_moved/needed_length) .. "%", 10, 22)
    
    if is_involume then
        love.graphics.setColor(0, 255, 0)
    else
        love.graphics.setColor(255, 0, 0)
    end
    if #pos >= 2 then
        love.graphics.print("Volume used: " .. string.format("%.0f", 100*(xmax - xmin)*(ymax - ymin)/maximal_volume) .. "%", 10, 34)
    else
        love.graphics.print("Volume used: 0%", 10, 34)
    end
    if is_in_time then
        love.graphics.setColor(0, 255, 0)
        love.graphics.print("Time left: " .. string.format("%.3f", maximal_time - stime) .. " seconds", 10 , 46)
    else
        love.graphics.setColor(255, 0, 0)
        love.graphics.print("Time left: none",10 , 46)
    end
    
    -- shaken wide and fast enough?
    if is_won then
        love.graphics.setColor(0, 255, 0)
        love.graphics.print("You won!", 10, 70)
    elseif is_involume and is_in_time then
        love.graphics.setColor(255, 255, 0)
        love.graphics.print("Keep moving!", 10, 70)
    else
        love.graphics.setColor(255, 0, 0)
        love.graphics.print("You lost!", 10, 70)
    end
    
    -- choose appropriate color
    if (is_involume and is_in_time) or is_won then
        love.graphics.setColor(0, 255, 0)
    else
        love.graphics.setColor(255, 0, 0)
    end
    
    -- draw the actual mouse movement
    for i,v in ipairs(pos) do
        -- draw every measured position
        love.graphics.point(v.x, v.y)
        
        -- draw the line of mouse movement
        if i > 1 then
            love.graphics.line(pos[i-1].x, pos[i-1].y, v.x, v.y)
        end
    end
    
    -- draw a rectangle around the mouse movement
    if #pos >= 2 then
        love.graphics.rectangle("line", xmin, ymin, xmax - xmin, ymax - ymin)
    end
end

function love.mousemoved(x, y, dx, dy)
    table.insert(pos, {x=x, y=y})
    length_moved = length_moved + math.sqrt(dx^2 + dy^2)
        
    -- update maximal positions
    if x < xmin then
        xmin = x
    end
    if x > xmax then
        xmax = x
    end
    if y < ymin then
        ymin = y
    end
    if y > ymax then
        ymax = y
    end
end

function love.mousepressed(x, y, button)
    reset_everything()
end

function love.keypressed(key, isrepeat)
    -- hotkeys to close the program
    if key == "q" or key == "escape" then
        love.event.quit()
    end
end

function reset_everything()
    -- x and y positions of the mouse for drawing
    pos = {}
    
    -- length of the mouses path
    length_moved = 0
    
    -- has the mousepointer stayed inside maximal_volume
    is_involume = true
    
    -- time being used to shake
    stime = 0
    
    -- has the player still time
    is_in_time = true
    
    -- is the game beeing won?
    is_won = false
    
    -- maximal positions
    xmin = love.graphics.getWidth()
    xmax = 0
    ymin = love.graphics.getHeight()
    ymax = 0
end
